const axios = require('axios');
const { getMetadata } = require('./metadata-store');
const { MOUNT_TARGET_SUFFIX, MOUNT_SOURCE_SUFFIX } = require('./consts');
const getLogger = require('./logger/logger');

const logger = getLogger();


async function sendAccessListRequest(source, target, aclServer, relationshipId) {
    try {
        const sourceCredentials = await getMetadata(`${relationshipId}.${MOUNT_SOURCE_SUFFIX}.cifs`);
        const targetCredentials = await getMetadata(`${relationshipId}.${MOUNT_TARGET_SUFFIX}.cifs`);
        const sourceToSend = Object.assign({}, source.cifs, sourceCredentials);
        const targetToSend = Object.assign({}, target.cifs, targetCredentials);
        logger.info(`Activating ACL server, sending to ${aclServer}, src:`, JSON.stringify(sourceToSend), 'trg:',
            JSON.stringify(targetToSend));
        return axios.post(`http://${aclServer}:8080/api/copy-acl`, {
            source: sourceToSend, target: targetToSend
        });
    } catch (e) {
        logger.error('Error on activating ACL server: ', e.response ? (e.response.data || e.response) : e);
        throw e;
    }
}

module.exports = sendAccessListRequest;
