const AWS = require('aws-sdk');
const proxy = require('proxy-agent');
const { SQS_DEFAULT_REGION } = require('../consts');
const getLogger = require('../logger/logger');

const logger = getLogger();

async function getAwsAccountId() {
    let httpOptions;
    if (process.env.http_proxy) {
        httpOptions = {
            agent: proxy(process.env.http_proxy)
        };
    }

    const sts = new AWS.STS({ region: SQS_DEFAULT_REGION, httpOptions });
    try {
        const callerIdentity = await sts.getCallerIdentity({}).promise();
        return callerIdentity.Account;
    } catch (err) {
        logger.warn('Caller identity is not available', err);
    }
}

module.exports = getAwsAccountId;
