const axios = require('axios');

async function getMetadata(url) {
    return new Promise(async (resolve, reject) => {
        try {
            const res = await axios.get(url, { headers: { Metadata: true } });
            resolve(res.data);
        } catch (err) {
            reject(err);
        }
    });
}

async function getAzureMetadata() {
    const [{ publicIpAddress }, { vmId }] = await Promise.all([
        getMetadata('http://169.254.169.254/metadata/instance/network/interface/0/ipv4/ipAddress/0/?api-version=2017-08-01'),
        getMetadata('http://169.254.169.254/metadata/instance/compute/?api-version=2017-08-01')
    ]);

    const azureMetadataObj = {
        instanceId: vmId
    };
    if (publicIpAddress) { azureMetadataObj.publicIp = publicIpAddress; }

    return azureMetadataObj;
}


module.exports = getAzureMetadata;
