const { set, get } = require('lodash');
const { writeFile, readFile, pathExists } = require('fs-extra');
const { queue } = require('async');
const getLogger = require('./logger/logger');

const logger = getLogger();
let META_DATA = {};
const INTERNAL_DISPATCHER_QUEUE = queue(async () => writeFile('./metadata.json', JSON.stringify(META_DATA, null, 4)), 1);

async function waitForEmptyQueue() {
    let interval;
    let timeout;
    const p1 = new Promise((resolve, reject) => {
        timeout = setTimeout(() => {
            clearInterval(interval);
            reject(new Error('An error occurred while converting metadata.json'));
        }, 10000);
    });
    const p2 = new Promise(resolve => {
        interval = setInterval(async () => {
            if (INTERNAL_DISPATCHER_QUEUE.idle()) {
                clearInterval(interval);
                clearTimeout(timeout);
                resolve();
            }
        }, 1000);
    });

    return Promise.race([p1, p2]);
}

function setAndDeleteCIFS(relationshipId, endpointObject, srcOrtrg) {
    if (Object.keys(endpointObject)[0] === 'cifs' && !get(META_DATA, `${relationshipId}.${srcOrtrg}.cifs`)) {
        set(META_DATA, `${relationshipId}.${srcOrtrg}.cifs`, get(META_DATA, `cifs['${endpointObject.cifs.host}']`));
        // TODO activate in the beginning of June
        // if (META_DATA.cifs) {
        //     delete META_DATA.cifs[endpointObject.cifs.host];
        // }
    }
}

function setAndDeleteSGWS(relationshipId, endpointObject, srcOrtrg) {
    if (Object.keys(endpointObject)[0] === 'sgws' && !get(META_DATA, `${relationshipId}.${srcOrtrg}.sgws`)) {
        set(META_DATA, `${relationshipId}.${srcOrtrg}.sgws`, get(META_DATA, `${srcOrtrg}.sgws['${endpointObject.sgws.host}:${endpointObject.sgws.port}']`));
        // TODO activate in the beginning of June
        // if (META_DATA[`${srcOrtrg}`]) {
        //     delete META_DATA[`${srcOrtrg}`].sgws[`${endpointObject.sgws.host}:${endpointObject.sgws.port}`];
        // }
    }
}

async function readMetaData() {
    if (await pathExists('./metadata.json')) {
        try {
            const data = await readFile('./metadata.json');
            META_DATA = JSON.parse(data);
        } catch (err) {
            logger.error('could not parse metadata.json', err);
        }
    }
}

async function convertMetadataJson(relationships) {
    await readMetaData();

    relationships.forEach(relationship => {
        const { relationshipId, source, target } = relationship;
        setAndDeleteCIFS(relationshipId, source, 'src');
        setAndDeleteCIFS(relationshipId, target, 'trg');

        setAndDeleteSGWS(relationshipId, source, 'src');
        setAndDeleteSGWS(relationshipId, target, 'trg');
    });

    // TODO activate in the beginning of June
    // if (isEmpty(META_DATA.cifs)) {
    //     delete META_DATA.cifs;
    // }
    // if (META_DATA.src && isEmpty(META_DATA.src.sgws)) {
    //     delete META_DATA.src;
    // }
    // if (META_DATA.trg && isEmpty(META_DATA.trg.sgws)) {
    //     delete META_DATA.trg;
    // }
    INTERNAL_DISPATCHER_QUEUE.push('');
    await waitForEmptyQueue();
}

module.exports = convertMetadataJson;
