const Joi = require('joi');
const bucketSchema = require('../schemas/bucket-schema');
const containerSchema = require('../schemas/container-schema');
const metadataSchema = require('../schemas/metadata-schema');
const {
    TYPE_SYMBOLIC, TYPE_FILE, TYPE_DIRECTORY, TYPE_OTHER, STATUS_FAILED, STATUS_SUCCEEDED, ACTION_DELETE, ACTION_COPY,
    ACTION_GRACE, ACTION_IGNORE, SCAN_STEP, TRANSFER_STEP, ACTION_DELETE_FAILED, LIST_STEP,
    ACTION_COPY_FAILED, ACTION_ABORT
} = require('../../lib/consts');
const sgwsBucketSchema = require('../schemas/sgws-bucket-schema');

const masterListBucketsResponse = Joi.object().keys({
    buckets: Joi.array().items(bucketSchema),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterListSgwsBucketsResponse = Joi.object().keys({
    buckets: Joi.array().items(sgwsBucketSchema),
    host: Joi.string(),
    port: Joi.string(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});


const masterListGcpBucketFoldersResponse = Joi.object().keys({
    bucket: bucketSchema,
    prefix: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});


const masterListGcpBucketsResponse = Joi.object().keys({
    buckets: Joi.array().items(bucketSchema),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});


const masterListSgwsBucketFoldersResponse = Joi.object().keys({
    bucket: Joi.string(),
    prefix: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    host: Joi.string(),
    port: Joi.string(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});
const masterListAzureBlobStorageContainersResponse = Joi.object().keys({
    containers: Joi.array().items(containerSchema),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterListExportsResponse = Joi.object().keys({
    exports: Joi.array().items(Joi.string()),
    host: Joi.string().required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterListExportFoldersResponse = Joi.object().keys({
    export: Joi.string().required(),
    host: Joi.string().required(),
    path: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterListBucketFoldersResponse = Joi.object().keys({
    bucket: bucketSchema,
    prefix: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterListAzureBlobStorageContainerFoldersResponse = Joi.object().keys({
    container: Joi.string().required(),
    prefix: Joi.string(),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterSharesResponse = Joi.object().keys({
    shares: Joi.array().items(Joi.string()),
    host: Joi.string().required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const masterShareFoldersResponse = Joi.object().keys({
    share: Joi.string().required(),
    host: Joi.string().required(),
    path: Joi.string(),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const metaDataAWS = Joi.object().keys({
    subnetId: Joi.string().required(),
    subnetCidrBlock: Joi.string().ip({ cidr: 'required' }).required(),
    vpcId: Joi.string().required(),
    vpcCidrBlock: Joi.string().required(),
    instanceId: Joi.string().required(),
    region: Joi.string().required(),
    availabilityZone: Joi.string().required(),
    awsAccountId: Joi.string().regex(/^[0-9]+$/, 'numbers').length(12).required(),
    proxy: Joi.string()
});

const metaDataGCP = Joi.object().keys({
    instanceId: Joi.string().required()
});

const metaDataAzure = Joi.object().keys({
    instanceId: Joi.string().required()
});

const placementMessageOnPrem = Joi.object().keys({
    version: Joi.string().required(),
    hostname: Joi.string().required(),
    privateIp: Joi.string().ip().required(),
    publicIp: Joi.string(),
    os: Joi.string().required(),
    platform: Joi.string().required(),
    release: Joi.string().required(),
    osTotalMem: Joi.number().positive().required(),
    node: Joi.string().required(),
    cpus: Joi.number().positive().required(),
    processMaxMem: Joi.number().positive().required(),
    awsAccountId: Joi.string().regex(/^[0-9]+$/, 'numbers').length(12),
    proxy: Joi.string(),
    certificate: Joi.string()
});

const placementMessageAWS = metaDataAWS.concat(placementMessageOnPrem);
const placementMessageGCP = metaDataGCP.concat(placementMessageOnPrem);
const placementMessageAzure = metaDataAzure.concat(placementMessageOnPrem);

const healthMessage = Joi.object().keys({
    transferrers: Joi.number().min(0).required(),
    scanners: Joi.number().min(0).required(),
    memory: Joi.object().keys({
        rss: Joi.number().positive().required(),
        heapTotal: Joi.number().positive().required(),
        heapUsed: Joi.number().positive().required(),
        external: Joi.number().required()
    }),
    cpuUsage: Joi.object().keys({
        user: Joi.number().positive().required(),
        system: Joi.number().positive().required()
    }),
    transferRate: Joi.number()
});

const statisticsScehma = Joi.object().keys({
    bytes: Joi.number().min(0).required(),
    items: Joi.number().min(0).required()
});

const countersSchema = Joi.object().keys({
    [TYPE_FILE]: statisticsScehma,
    [TYPE_DIRECTORY]: statisticsScehma,
    [TYPE_SYMBOLIC]: statisticsScehma,
    [TYPE_OTHER]: statisticsScehma
}).or(TYPE_FILE, TYPE_DIRECTORY, TYPE_SYMBOLIC, TYPE_OTHER);

const scanAndTransferResponse = Joi.object().keys({
    relationshipId: Joi.string().required(),
    [SCAN_STEP]: Joi.object().keys({
        [STATUS_FAILED]: countersSchema,
        [ACTION_DELETE]: countersSchema,
        [ACTION_COPY]: countersSchema,
        [ACTION_GRACE]: countersSchema,
        [ACTION_IGNORE]: countersSchema,
        [ACTION_ABORT]: countersSchema
    }).or(ACTION_DELETE, ACTION_COPY, ACTION_GRACE, ACTION_IGNORE, STATUS_FAILED, ACTION_ABORT),
    [TRANSFER_STEP]: Joi.object().keys({
        [ACTION_DELETE]: countersSchema,
        [ACTION_COPY]: countersSchema,
        [ACTION_DELETE_FAILED]: countersSchema,
        [ACTION_COPY_FAILED]: countersSchema,
        [ACTION_ABORT]: countersSchema
    }).or(ACTION_DELETE, ACTION_COPY, ACTION_ABORT, ACTION_DELETE_FAILED, ACTION_COPY_FAILED),
    [LIST_STEP]: Joi.object().keys({
        [LIST_STEP]: countersSchema,
        [STATUS_FAILED]: countersSchema
    })
}).or(SCAN_STEP, TRANSFER_STEP, LIST_STEP);

const masterMetadataResponseSchema = Joi.object().keys({
    targetDataBrokerId: Joi.string().required(),
    metadata: metadataSchema,
    status: Joi.string().valid([STATUS_SUCCEEDED])
});

const sqsMasterOutgoingMessagesSchemes = {
    buckets: masterListBucketsResponse,
    'bucket-folders': masterListBucketFoldersResponse,
    'sgws-buckets': masterListSgwsBucketsResponse,
    'gcp-buckets': masterListGcpBucketsResponse,
    'gcp-bucket-folders': masterListGcpBucketFoldersResponse,
    'sgws-buckets-folders': masterListSgwsBucketFoldersResponse,
    exports: masterListExportsResponse,
    'export-folders': masterListExportFoldersResponse,
    shares: masterSharesResponse,
    'share-folders': masterShareFoldersResponse,
    'azure-blob-storage-containers': masterListAzureBlobStorageContainersResponse,
    'azure-blob-storage-container-folders': masterListAzureBlobStorageContainerFoldersResponse,
    'placement-aws': placementMessageAWS,
    'placement-gcp': placementMessageGCP,
    'placement-azure': placementMessageAzure,
    'placement-on-prem': placementMessageOnPrem,
    health: healthMessage,
    statistics: scanAndTransferResponse,
    metadata: masterMetadataResponseSchema
};

module.exports = sqsMasterOutgoingMessagesSchemes;
