const Joi = require('joi');
const { NFS_VERSIONS, CIFS_VERSIONS } = require('../../lib/consts');

const endPointSchema = Joi.object().keys({
    cifs: Joi.object().keys({
        host: Joi.string().required(),
        share: Joi.string().required(),
        path: Joi.string(),
        username: Joi.string(),
        password: Joi.string(),
        workgroup: Joi.string(),
        version: Joi.number().valid(CIFS_VERSIONS),
        aclServer: Joi.string()
    }),
    nfs: Joi.object().keys({
        host: Joi.string().required(),
        export: Joi.string().required(),
        path: Joi.string(),
        version: Joi.number().valid(NFS_VERSIONS)
    }),
    snfs: Joi.object().keys({
        serverIP: Joi.string().required(),
        host: Joi.string().required(),
        export: Joi.string().required(),
        path: Joi.string(),
        version: Joi.number().valid(NFS_VERSIONS)
    }),
    local: Joi.object().keys({ path: Joi.string().required() }),
    efs: Joi.object().keys({
        host: Joi.string().required(),
        export: Joi.string().required(),
        path: Joi.string()
    }),
    s3: Joi.object().keys({
        region: Joi.string().required(),
        bucket: Joi.string().required(),
        path: Joi.string().allow(''),
        storageClass: Joi.string().valid(['STANDARD', 'STANDARD_IA', 'ONEZONE_IA', 'INTELLIGENT_TIERING', 'GLACIER']),
        encryption: Joi.object().keys({
            kms: Joi.object().keys({
                keyId: Joi.string().required()
            })
        }),
        tags: Joi.array().items(Joi.object().keys({
            Key: Joi.string().required(),
            Value: Joi.string().required()
        }))
    }),
    sgws: Joi.object().keys({
        host: Joi.string().required(),
        bucket: Joi.string().required(),
        path: Joi.string().allow(''),
        port: Joi.string(),
        accessKey: Joi.string(),
        secretKey: Joi.string(),
        tags: Joi.array().items(Joi.object().keys({
            Key: Joi.string().required(),
            Value: Joi.string().required()
        }))
    }),
    azure: Joi.object().keys({
        container: Joi.string().required(),
        storageAccountName: Joi.string().required(),
        storageAccountConnectionString: Joi.string(),
        path: Joi.string(),
        tags: Joi.array().items(Joi.object().keys({
            Key: Joi.string().required(),
            Value: Joi.string().required()
        })),
        blobTier: Joi.string().valid(['HOT', 'COOL', 'ARCHIVE'])
    }),
    gcp: Joi.object().keys({
        bucket: Joi.string().required(),
        path: Joi.string().allow(''),
        tags: Joi.array().items(Joi.object().keys({
            Key: Joi.string().required(),
            Value: Joi.string().required()
        })),
        projectId: Joi.string(),
        clientEmail: Joi.string().email(),
        privateKey: Joi.string()
    })
}).xor('cifs', 'nfs', 'efs', 's3', 'sgws', 'local', 'azure', 'gcp', 'snfs');

module.exports = endPointSchema;
