const Joi = require('joi');
const getLogger = require('../../lib/logger/logger');

const logger = getLogger();

function validateMessage(messageBody, channel, schema) {
    if (!schema) {
        throw new Error(`validateMessage schema is not defined for message ${JSON.stringify(messageBody, null, 4)}, channel ${channel}`);
    } else {
        const { error } = Joi.validate(messageBody, schema, { abortEarly: false, allowUnknown: true });
        if (error) {
            const errors = error.details.map(detail => detail.message);
            logger.error(`!!!~ ERROR ~!!! Validation error, ${channel} message: ${JSON.stringify(messageBody, null, 4)}`, errors);
            throw new Error(errors);
        }
        return true;
    }
}

module.exports = validateMessage;
