const Joi = require('joi');
const {
    TYPE_SYMBOLIC, TYPE_FILE, TYPE_DIRECTORY, TYPE_OTHER, STATUS_FAILED, STATUS_SUCCEEDED,
    STATUS_IGNORED, ACTION_DELETE, ACTION_COPY, ACTION_GRACE, ACTION_IGNORE, SCAN_STEP, TRANSFER_STEP, LIST_STEP,
    INITIALIZE_STEP
} = require('../../lib/consts');

const copyStatisticsSchema = Joi.object().keys({
    status: Joi.string().valid([STATUS_SUCCEEDED, STATUS_FAILED, STATUS_IGNORED]).required(),
    step: Joi.string()
        .when('status', { is: STATUS_IGNORED, then: Joi.string().valid(SCAN_STEP), otherwise: Joi.valid(SCAN_STEP, TRANSFER_STEP, LIST_STEP) }).required(),
    relationshipId: Joi.string().required(),
    bytes: Joi.number().integer().required(),
    items: Joi.number().integer(),
    objectType: Joi.string().valid(TYPE_FILE, TYPE_DIRECTORY, TYPE_SYMBOLIC)
        .when('status', { is: STATUS_IGNORED, then: Joi.string().valid(TYPE_OTHER) }).required(),
    targetRequiredAction: Joi.string().valid(ACTION_COPY, ACTION_DELETE, ACTION_GRACE, STATUS_FAILED, LIST_STEP)
        .when('status', { is: STATUS_IGNORED, then: Joi.string().valid(ACTION_IGNORE) })
        .required(),
    path: Joi.string(), // should add validation for case of failure and not step scan, should be required
    reason: Joi.object().when('status', { is: STATUS_FAILED, then: Joi.required() }),
    attributes: Joi.object().required()
});

const failedSchema = Joi.object().keys({
    step: Joi.string().valid(INITIALIZE_STEP).required(),
    relationshipId: Joi.string(),
    status: Joi.string().valid([STATUS_FAILED]).required(),
    reason: Joi.any().required()
});

module.exports = { copyStatisticsSchema, failedSchema };
