const Joi = require('joi');
const { cloneDeep } = require('lodash');
const sqsMasterIncomingMessagesSchemes = require('../../../src/messages/master/incoming-messages');

describe('masterCopySchema validations ', () => {
    const masterCopySchema = sqsMasterIncomingMessagesSchemes.copy;
    const validMasterCopy = {
        relationshipId: '1',
        source: {
            cifs: {
                host: '1.1.1.1',
                share: 'share',
                username: 'username',
                password: 'password'
            }
        },
        target: {
            s3: {
                bucket: 'bucket',
                region: 'us-east-1',
                encryption: {
                    kms: {
                        keyId: '11111'
                    }
                }
            }
        },
        scannerQueue: 'https://sqs.us-east-1.amazonaws.com/718273455463/scanner',
        transferrerQueue: 'https://sqs.us-east-1.amazonaws.com/718273455463/transferrer',
        grace: 0,
        deleteOnTarget: true
    };

    test('masterCopySchema valid command', () => expect(Joi.validate(validMasterCopy, masterCopySchema).error).toBeNull());

    test('masterCopySchema gracePeriod not valid', () => {
        const invalidMasterCopy = cloneDeep(validMasterCopy);
        invalidMasterCopy.gracePeriod = 1.1;
        expect(Joi.validate(invalidMasterCopy, masterCopySchema).error.details[0])
            .toEqual(expect.objectContaining({ message: '"gracePeriod" is not allowed' }));
    });

    test('masterCopySchema source endpoint contains more than one protocol', () => {
        const invalidMasterCopy = cloneDeep(validMasterCopy);
        invalidMasterCopy.source.nfs = {
            host: '2.2.2.2',
            version: 4.1,
            export: 'export'
        };
        expect(Joi.validate(invalidMasterCopy, masterCopySchema).error.details[0])
            .toEqual(expect.objectContaining({ message: '"value" contains a conflict between exclusive peers [cifs, nfs, efs, s3, sgws, local, azure, gcp, snfs]' }));
    });

    test('masterCopySchema grace period is negative', () => {
        const invalidMasterCopy = cloneDeep(validMasterCopy);
        invalidMasterCopy.grace = -1;
        expect(Joi.validate(invalidMasterCopy, masterCopySchema).error.details[0])
            .toEqual(expect.objectContaining({ message: '"grace" must be larger than or equal to 0' }));
    });
});
