const { find, _ } = require('lodash');
const { COPY_TABLE, isNetappEntry } = require('../../src/workers/transferrer');

describe('find in copy table', () => {
    test('find match for local to nfs directory', () => {
        const source = { local: { path: '/dev-ops' } };
        const target = { nfs: { path: 'localFromDataBroker', host: '10.60.13.47', export: '/exp/trg2' } };
        const entry = {
            path: 'test', type: 'DIRECTORY', size: 0, mode: 16877, uid: 0, gid: 0
        };
        const match = find(COPY_TABLE, {
            source: _(source).keys().head(),
            target: _(target).keys().head(),
            type: entry.type
        });
        expect(match).not.toBeNull();
    });
});

describe('is netapp entry', () => {
    test('match - regular', () => expect(isNetappEntry('.france_6YmSIj3_netapp')).toBeTruthy());
    test('match - starts with underscore', () => expect(isNetappEntry('._france_6YmSIj3_netapp')).toBeTruthy());
    test('match - starts with dot', () => expect(isNetappEntry('..france_6YmSIj3_netapp')).toBeTruthy());
    test('match - ends with underscore', () => expect(isNetappEntry('.france__6YmSIj3_netapp')).toBeTruthy());
    test('match - with dot', () => expect(isNetappEntry('.fra.nce_6YmSIj3_netapp')).toBeTruthy());
    test('match - with dollar', () => expect(isNetappEntry('.fra$nce_6YmSIj3_netapp')).toBeTruthy());
    test('match - hebrew', () => expect(isNetappEntry('.צרפת_asdF34f_netapp')).toBeTruthy());
    test('mismatch - regular', () => expect(isNetappEntry('hello.txt')).toBeFalsy());
    test('mismatch - regular hebrew', () => expect(isNetappEntry('שלום.txt')).toBeFalsy());
    test('mismatch - long sufix', () => expect(isNetappEntry('.france_6YmSIj3_netapplong')).toBeFalsy());
    test('mismatch - wrong sufix', () => expect(isNetappEntry('.france_6YmSIj3netapp')).toBeFalsy());
    test('mismatch - wrong random length', () => expect(isNetappEntry('.france_a12_netapp')).toBeFalsy());
    test('mismatch - wrong random type', () => expect(isNetappEntry('.france_6Ym!Ij3_netapp')).toBeFalsy());
});
