const { resolve } = require('path');
const nodeExternals = require('webpack-node-externals');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');

const BUILD = 'build';

module.exports = {
    target: 'node',
    mode: 'production',
    node: {
        __dirname: false
    },
    entry: {
        app: './src/index.js',
        master: './src/workers/master.js',
        scanner: './src/workers/scanner.js',
        transferrer: './src/workers/transferrer.js',
        server: './src/workers/server.js',
        'kinesis-stream': './src/lib/logger/appenders/kinesis-stream.js'
    },
    output: {
        path: resolve(__dirname, BUILD),
        filename: '[name].js',
        libraryTarget: 'commonjs-module'
    },
    externals: [nodeExternals()],
    module: {
        rules: [
            {
                enforce: 'pre',
                test: /\.js?$/,
                exclude: /node_modules/,
                loader: 'eslint-loader',
                options: {
                    quiet: true,
                    failOnError: true,
                    failOnWarning: true,
                    emitError: false,
                    emitWarning: false
                }
            },
            {
                test: /\.json$/,
                loader: 'json'
            }
        ]
    },
    optimization: {
        minimizer: [new TerserPlugin()]
    },
    plugins: [
        new CleanWebpackPlugin(),
        new CopyWebpackPlugin([
            { from: 'package-lock.json', to: 'package-lock.json' },
            { from: 'package.json', to: 'package.json' },
            { from: './config/default.json', to: './config/default.json' },
            {
                from: './config/log4js.json',
                to: './config/log4js.json',
                transform: content => {
                    const log4js = JSON.parse(content.toString());
                    log4js.appenders['kinesis-stream'].type = 'kinesis-stream';
                    return JSON.stringify(log4js, null, 4);
                }
            }
        ])
    ],
    devtool: false
};
